@@ -83,6 +83,9 @@ struct mqtt3_config {
 	char *acl_file;
 	bool allow_anonymous;
 	bool allow_duplicate_messages;
+	bool allow_zero_length_clientid;
+	char *auto_id_prefix;
+	int auto_id_prefix_len;
 	int autosave_interval;
 	bool autosave_on_changes;
 	char *clientid_prefixes;
@@ -171,10 +174,11 @@ struct _mosquitto_unpwd{
 };
 
 struct _mosquitto_acl{
-	struct _mosquitto_acl *child;
 	struct _mosquitto_acl *next;
 	char *topic;
 	int access;
+	int ucount;
+	int ccount;
 };
 
 struct _mosquitto_acl_user{
@@ -267,6 +271,8 @@ struct _mqtt3_bridge{
 	time_t restart_t;
 	char *username;
 	char *password;
+	char *local_username;
+	char *local_password;
 	bool notifications;
 	char *notification_topic;
 	enum mosquitto_bridge_start_type start_type;
