@@ -83,7 +83,6 @@ int mqtt3_socket_accept(struct mosquitto_db *db, int listensock)
 	int new_sock = -1;
 	struct mosquitto **tmp_contexts = NULL;
 	struct mosquitto *new_context;
-	int opt = 1;
 #ifdef WITH_TLS
 	BIO *bio;
 	int rc;
@@ -102,20 +101,9 @@ int mqtt3_socket_accept(struct mosquitto_db *db, int listensock)
 	g_socket_connections++;
 #endif
 
-#ifndef WIN32
-	/* Set non-blocking */
-	opt = fcntl(new_sock, F_GETFL, 0);
-	if(opt == -1 || fcntl(new_sock, F_SETFL, opt | O_NONBLOCK) == -1){
-		/* If either fcntl fails, don't want to allow this client to connect. */
-		close(new_sock);
-		return -1;
-	}
-#else
-	if(ioctlsocket(new_sock, FIONBIO, &opt)){
-		closesocket(new_sock);
+	if(_mosquitto_socket_nonblock(new_sock)){
 		return INVALID_SOCKET;
 	}
-#endif
 
 #ifdef WITH_WRAP
 	/* Use tcpd / libwrap to determine whether a connection is allowed. */
@@ -139,20 +127,63 @@ int mqtt3_socket_accept(struct mosquitto_db *db, int listensock)
 			for(j=0; j<db->config->listeners[i].sock_count; j++){
 				if(db->config->listeners[i].socks[j] == listensock){
 					new_context->listener = &db->config->listeners[i];
+					new_context->listener->client_count++;
 					break;
 				}
 			}
 		}
 		if(!new_context->listener){
-			COMPAT_CLOSE(new_sock);
+			mqtt3_context_cleanup(NULL, new_context, true);
 			return -1;
 		}
 
-		if(new_context->listener->max_connections > 0 && new_context->listener->client_count >= new_context->listener->max_connections){
+		if(new_context->listener->max_connections > 0 && new_context->listener->client_count > new_context->listener->max_connections){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client connection from %s denied: max_connections exceeded.", new_context->address);
-			COMPAT_CLOSE(new_sock);
+			mqtt3_context_cleanup(NULL, new_context, true);
 			return -1;
 		}
+
+#ifdef WITH_TLS
+		/* TLS init */
+		for(i=0; i<db->config->listener_count; i++){
+			for(j=0; j<db->config->listeners[i].sock_count; j++){
+				if(db->config->listeners[i].socks[j] == listensock){
+					if(db->config->listeners[i].ssl_ctx){
+						new_context->ssl = SSL_new(db->config->listeners[i].ssl_ctx);
+						if(!new_context->ssl){
+							mqtt3_context_cleanup(NULL, new_context, true);
+							return -1;
+						}
+						SSL_set_ex_data(new_context->ssl, tls_ex_index_context, new_context);
+						SSL_set_ex_data(new_context->ssl, tls_ex_index_listener, &db->config->listeners[i]);
+						new_context->want_write = true;
+						bio = BIO_new_socket(new_sock, BIO_NOCLOSE);
+						SSL_set_bio(new_context->ssl, bio, bio);
+						rc = SSL_accept(new_context->ssl);
+						if(rc != 1){
+							rc = SSL_get_error(new_context->ssl, rc);
+							if(rc == SSL_ERROR_WANT_READ){
+								/* We always want to read. */
+							}else if(rc == SSL_ERROR_WANT_WRITE){
+								new_context->want_write = true;
+							}else{
+								e = ERR_get_error();
+								while(e){
+									_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE,
+											"Client connection from %s failed: %s.",
+											new_context->address, ERR_error_string(e, ebuf));
+									e = ERR_get_error();
+								}
+								mqtt3_context_cleanup(NULL, new_context, true);
+								return -1;
+							}
+						}
+					}
+				}
+			}
+		}
+#endif
+
 		_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "New connection from %s on port %d.", new_context->address, new_context->listener->port);
 		for(i=0; i<db->context_count; i++){
 			if(db->contexts[i] == NULL){
@@ -169,54 +200,11 @@ int mqtt3_socket_accept(struct mosquitto_db *db, int listensock)
 			}else{
 				// Out of memory
 				mqtt3_context_cleanup(NULL, new_context, true);
+				return -1;
 			}
 		}
 		// If we got here then the context's DB index is "i" regardless of how we got here
 		new_context->db_index = i;
-		new_context->listener->client_count++;
-
-#ifdef WITH_TLS
-		/* TLS init */
-		for(i=0; i<db->config->listener_count; i++){
-			for(j=0; j<db->config->listeners[i].sock_count; j++){
-				if(db->config->listeners[i].socks[j] == listensock){
-					if(db->config->listeners[i].ssl_ctx){
-						new_context->ssl = SSL_new(db->config->listeners[i].ssl_ctx);
-						if(!new_context->ssl){
-							COMPAT_CLOSE(new_sock);
-							return -1;
-						}
-						SSL_set_ex_data(new_context->ssl, tls_ex_index_context, new_context);
-						SSL_set_ex_data(new_context->ssl, tls_ex_index_listener, &db->config->listeners[i]);
-						new_context->want_read = true;
-						new_context->want_write = true;
-						bio = BIO_new_socket(new_sock, BIO_NOCLOSE);
-						SSL_set_bio(new_context->ssl, bio, bio);
-						rc = SSL_accept(new_context->ssl);
-						if(rc != 1){
-							rc = SSL_get_error(new_context->ssl, rc);
-							if(rc == SSL_ERROR_WANT_READ){
-								new_context->want_read = true;
-							}else if(rc == SSL_ERROR_WANT_WRITE){
-								new_context->want_write = true;
-							}else{
-								e = ERR_get_error();
-								while(e){
-									_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE,
-											"Client connection from %s failed: %s.",
-											new_context->address, ERR_error_string(e, ebuf));
-									e = ERR_get_error();
-								}
-								COMPAT_CLOSE(new_sock);
-								SSL_free(new_context->ssl);
-								new_context->ssl = NULL;
-							}
-						}
-					}
-				}
-			}
-		}
-#endif
 
 #ifdef WITH_WRAP
 	}
@@ -285,7 +273,6 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 	struct addrinfo hints;
 	struct addrinfo *ainfo, *rp;
 	char service[10];
-	int opt = 1;
 #ifndef WIN32
 	int ss_opt = 1;
 #else
@@ -295,8 +282,9 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 	int rc;
 	X509_STORE *store;
 	X509_LOOKUP *lookup;
+	int ssl_options = 0;
 #endif
-	char err[256];
+	char buf[256];
 
 	if(!listener) return MOSQ_ERR_INVAL;
 
@@ -322,8 +310,8 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 
 		sock = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
 		if(sock == -1){
-			strerror_r(errno, err, 256);
-			_mosquitto_log_printf(NULL, MOSQ_LOG_WARNING, "Warning: %s", err);
+			strerror_r(errno, buf, 256);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_WARNING, "Warning: %s", buf);
 			continue;
 		}
 		listener->sock_count++;
@@ -341,32 +329,20 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 		ss_opt = 1;
 		setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, &ss_opt, sizeof(ss_opt));
 
-
-#ifndef WIN32
-		/* Set non-blocking */
-		opt = fcntl(sock, F_GETFL, 0);
-		if(opt == -1 || fcntl(sock, F_SETFL, opt | O_NONBLOCK) == -1){
-			/* If either fcntl fails, don't want to allow this client to connect. */
-			COMPAT_CLOSE(sock);
+		if(_mosquitto_socket_nonblock(sock)){
 			return 1;
 		}
-#else
-		if(ioctlsocket(sock, FIONBIO, &opt)){
-			COMPAT_CLOSE(sock);
-			return 1;
-		}
-#endif
 
 		if(bind(sock, rp->ai_addr, rp->ai_addrlen) == -1){
-			strerror_r(errno, err, 256);
-			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s", err);
+			strerror_r(errno, buf, 256);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s", buf);
 			COMPAT_CLOSE(sock);
 			return 1;
 		}
 
 		if(listen(sock, 100) == -1){
-			strerror_r(errno, err, 256);
-			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s", err);
+			strerror_r(errno, buf, 256);
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s", buf);
 			COMPAT_CLOSE(sock);
 			return 1;
 		}
@@ -385,24 +361,36 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 			}else if(!strcmp(listener->tls_version, "tlsv1.1")){
 				listener->ssl_ctx = SSL_CTX_new(TLSv1_1_server_method());
 			}else if(!strcmp(listener->tls_version, "tlsv1")){
-				listener->ssl_ctx = SSL_CTX_new(TLSv1_server_method());
+				listener->ssl_ctx = SSL_CTX_new(SSLv23_server_method());
 			}
 #else
-			listener->ssl_ctx = SSL_CTX_new(TLSv1_server_method());
+			listener->ssl_ctx = SSL_CTX_new(SSLv23_server_method());
 #endif
 			if(!listener->ssl_ctx){
 				_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Unable to create TLS context.");
 				COMPAT_CLOSE(sock);
 				return 1;
 			}
-#if OPENSSL_VERSION_NUMBER >= 0x10000000
+
+			/* Don't accept SSLv2 or SSLv3 */
+			ssl_options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
+#ifdef SSL_OP_NO_COMPRESSION
 			/* Disable compression */
-			SSL_CTX_set_options(listener->ssl_ctx, SSL_OP_NO_COMPRESSION);
+			ssl_options |= SSL_OP_NO_COMPRESSION;
 #endif
+#ifdef SSL_OP_CIPHER_SERVER_PREFERENCE
+			/* Server chooses cipher */
+			ssl_options |= SSL_OP_CIPHER_SERVER_PREFERENCE;
+#endif
+			SSL_CTX_set_options(listener->ssl_ctx, ssl_options);
+
 #ifdef SSL_MODE_RELEASE_BUFFERS
 			/* Use even less memory per SSL connection. */
 			SSL_CTX_set_mode(listener->ssl_ctx, SSL_MODE_RELEASE_BUFFERS);
 #endif
+			snprintf(buf, 256, "mosquitto-%d", listener->port);
+			SSL_CTX_set_session_id_context(listener->ssl_ctx, (unsigned char *)buf, strlen(buf));
+
 			if(listener->ciphers){
 				rc = SSL_CTX_set_cipher_list(listener->ssl_ctx, listener->ciphers);
 				if(rc == 0){
@@ -410,6 +398,13 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 					COMPAT_CLOSE(sock);
 					return 1;
 				}
+			}else{
+				rc = SSL_CTX_set_cipher_list(listener->ssl_ctx, "DEFAULT:!aNULL:!eNULL:!LOW:!EXPORT:!SSLv2:@STRENGTH");
+				if(rc == 0){
+					_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Unable to set TLS ciphers. Check cipher list \"%s\".", listener->ciphers);
+					COMPAT_CLOSE(sock);
+					return 1;
+				}
 			}
 			rc = SSL_CTX_load_verify_locations(listener->ssl_ctx, listener->cafile, listener->capath);
 			if(rc == 0){
